@@ -19,11 +19,17 @@
 
 /* ~~~ WeatherLocationListModel ~~~ */
 WeatherLocationListModel::WeatherLocationListModel(QObject *parent)
-    : QAbstractListModel(parent)
+    : QObject{parent}
 {
     load();
 }
 
+WeatherLocationListModel *WeatherLocationListModel::inst()
+{
+    static WeatherLocationListModel singleton;
+    return &singleton;
+}
+
 void WeatherLocationListModel::load()
 {
     // load locations from kconfig
@@ -35,25 +41,37 @@ void WeatherLocationListModel::load()
             m_locations.push_back(location_ptr);
     }
 
-    auto i{0};
+    // sort locations by index, correcting any issues with the stored index
+
+    QList<WeatherLocation *> sorted, unsorted;
+    for (int i = 0; i < (int)m_locations.size(); ++i) {
+        sorted.push_back(nullptr);
+    }
+
+    // loop through the initial locations and fill in the indicies in sorted
     for (auto loc : m_locations) {
         auto index = loc->index();
-        while (index != i) {
-            if (index != -1) {
-                if (index >= (int)m_locations.size()) {
-                    qDebug() << "invalid index, please delete config(~/.config/kweather/kweatherrc)";
-                    abort();
-                } else {
-                    std::swap(m_locations[i], m_locations[index]);
-                    index = m_locations[i]->index();
-                }
-            } else {
-                loc->saveOrder(i);
-                index = i;
+
+        if (index < 0 || index >= (int)sorted.size() || sorted[index] != nullptr) {
+            unsorted.push_back(loc);
+        } else {
+            sorted[index] = loc;
+        }
+    }
+    // add unsorted locations in positions unfilled
+    for (auto loc : unsorted) {
+        for (int i = 0; i < (int)sorted.size(); ++i) {
+            if (!sorted[i]) {
+                sorted[i] = loc;
+                break;
             }
         }
-        i++;
     }
+    // move into original array
+    for (int i = 0; i < (int)m_locations.size(); ++i) {
+        m_locations[i] = sorted[i];
+    }
+    Q_EMIT locationsChanged();
 }
 
 void WeatherLocationListModel::saveOrder()
@@ -65,45 +83,15 @@ void WeatherLocationListModel::saveOrder()
     }
 }
 
-int WeatherLocationListModel::rowCount(const QModelIndex &parent) const
-{
-    Q_UNUSED(parent);
-    return m_locations.size();
-}
-
-QVariant WeatherLocationListModel::data(const QModelIndex &index, int role) const
-{
-    if (!index.isValid() || index.row() >= static_cast<int>(m_locations.size()) || index.row() < 0) {
-        return {};
-    }
-    if (role == Roles::LocationRole) {
-        return QVariant::fromValue(m_locations.at(index.row()));
-    }
-    return {};
-}
-
-QHash<int, QByteArray> WeatherLocationListModel::roleNames() const
-{
-    return {{Roles::LocationRole, "location"}};
-}
-
-void WeatherLocationListModel::updateUi()
-{
-    Q_EMIT dataChanged(createIndex(0, 0), createIndex(m_locations.size() - 1, 0));
-    for (const auto &l : qAsConst(m_locations)) {
-        Q_EMIT l->propertyChanged();
-    }
-}
-
 void WeatherLocationListModel::insert(int index, WeatherLocation *weatherLocation)
 {
     if ((index < 0) || (index > static_cast<int>(m_locations.size())))
         return;
 
     QQmlEngine::setObjectOwnership(weatherLocation, QQmlEngine::CppOwnership);
-    beginInsertRows(QModelIndex(), index, index);
     m_locations.insert(m_locations.begin() + index, weatherLocation);
-    endInsertRows();
+
+    Q_EMIT locationsChanged();
 
     saveOrder();
     weatherLocation->save();
@@ -114,35 +102,24 @@ void WeatherLocationListModel::remove(int index)
     if ((index < 0) || (index >= static_cast<int>(m_locations.size())))
         return;
 
-    beginRemoveRows(QModelIndex(), index, index);
     auto location = m_locations.at(index);
     m_locations.erase(m_locations.begin() + index);
     location->deleteConfig();
     location->deleteLater();
-    endRemoveRows();
+
+    Q_EMIT locationsChanged();
+
     saveOrder();
 }
 
-WeatherLocation *WeatherLocationListModel::get(int index)
-{
-    if ((index < 0) || (index >= static_cast<int>(m_locations.size())))
-        return {};
-
-    return m_locations.at(index);
-}
-
 void WeatherLocationListModel::move(int oldIndex, int newIndex)
 {
     int locationsSize = m_locations.size();
     if (oldIndex < 0 || oldIndex >= locationsSize || newIndex < 0 || newIndex >= locationsSize)
         return;
 
-    if (oldIndex < newIndex)
-        ++newIndex;
-
-    beginMoveRows(QModelIndex(), oldIndex, oldIndex, QModelIndex(), newIndex);
     std::iter_swap(m_locations.begin() + oldIndex, m_locations.begin() + newIndex);
-    endMoveRows();
+    Q_EMIT locationsChanged();
 
     saveOrder();
 }
@@ -172,9 +149,9 @@ void WeatherLocationListModel::requestCurrentLocation()
         geoPtr = new KWeatherCore::LocationQuery(this);
 
     geoPtr->locate();
-    //    // failure
+    // failure
     connect(geoPtr, &KWeatherCore::LocationQuery::queryError, this, &WeatherLocationListModel::networkErrorCreatingDefault);
-    //    // success
+    // success
     connect(geoPtr, &KWeatherCore::LocationQuery::located, this, &WeatherLocationListModel::addCurrentLocation);
 }
 
@@ -185,3 +162,8 @@ void WeatherLocationListModel::addCurrentLocation(const KWeatherCore::LocationQu
     insert(0, location);
     Q_EMIT successfullyCreatedDefault();
 }
+
+QList<WeatherLocation *> &WeatherLocationListModel::locations()
+{
+    return m_locations;
+}
